require 'rails_helper'

RSpec.describe Hermes::HousingApplication, type: :model do 
  describe 'edit with change status' do
    let!(:applicant) { create(:applicant) }
    let!(:housing_offer1) do
      create(
        :housing_offer,
        typology: 'T2',
        external_ref: 'blabli'
      )
    end

    let!(:housing_request1) do
      create(
        :housing_request,
        is_active: true,
        main_applicant: applicant
      )
    end
    let!(:housing_application1) do
      create(
        :housing_application,
        housing_offer_id: housing_offer1.id,
        housing_request_id: housing_request1.id,
        status: 'Inscription CAL'
      )
    end
    let!(:housing_application2) do
      create(
        :housing_application,
        housing_offer_id: housing_offer1.id,
        housing_request_id: housing_request1.id,
        status: 'Inscription CAL'
      )
    end

    it 'trigger sopra flux' do
      housing_application1.status = 'Visite effectuée'
      after_create_res = housing_application1.after_save_actions
      expect(after_create_res[:exectuted]).to eq(true)
    end
    
    it 'increment is visible in housing_request_serializer' do
       json =    Hermes::HousingRequestSerializer.new(
        housing_request1.reload,
       # scope: scope,
        root: false
      ).to_json
      expect(JSON.parse(json)['applications_nb']).to eq(2)
    end
  end

  describe 'Cancelation' do
    let!(:canceler_applicant) { create(:applicant) }
    let!(:housing_offer1) { create(:housing_offer, offer_status: 'LOGEMENT PUBLIE', typology: 'T1') }
    let!(:housing_offer2) { create(:housing_offer, offer_status: 'LOGEMENT PUBLIE', typology: 'T2') }
    let!(:housing_request1) { create(:housing_request, is_active: true, main_applicant: canceler_applicant) }
    let!(:housing_request2) { create(:housing_request, is_active: true, main_applicant: canceler_applicant) }
    let(:housing_application1) do
      create(
        :housing_application,
        housing_offer_id: housing_offer1.id,
        housing_request_id: housing_request1.id,
        status: 'Inscription CAL'
      )
    end
    let(:housing_application2) do
      create(
        :housing_application,
        housing_offer_id: housing_offer2.id,
        housing_request_id: housing_request2.id,
        status: 'Inscription CAL'
      )
    end

    before do
      drain_all_jobs
    end

    it 'trigger cancelation' do
      housing_application1.update!(status: 'DESISTEMENT', visit_status: 'VISITE EFFECTUEE')
      housing_application2.update!(status: 'DESISTEMENT', visit_status: 'VISITE EFFECTUEE')
      canceler_applicant.reload
      expect(canceler_applicant.canceled_application_nb).to eq(2)
      expect(enqueued_jobs.last[:job]).to eq(UpdateScoreJob)
      expect(enqueued_jobs.last[:args]).to eq(['housing_request', [housing_offer1.id.to_s, housing_offer2.id.to_s], canceler_applicant.id.to_s])
    end
  end

  describe 'Hooks' do
    let!(:lessor_user) {create(:user, first_name: 'Jaques', last_name: 'Dupont', email: 'jaques.dupont@gmail.com', roles: ['Bailleur'])}
    let!(:lessor) {create(:lessor, user_id: lessor_user.id.to_s)}
    let!(:housing_offer) {create(:housing_offer, lessor: lessor)}
    let!(:applicant) { create(:applicant) }
    let!(:applicant2) { create(:applicant) }
    let!(:housing_request) {create(:housing_request, main_applicant_id: applicant.id, is_active: true, applicant_ids: [applicant2.id])}
    let!(:housing_application) {create(:housing_application, housing_request: housing_request, housing_offer: housing_offer)}

    before do
      applicant.reload
      applicant2.reload
    end

    it 'setup lessor as readers' do
      expect(housing_application.readers).to include(lessor_user.id.to_s)
      expect(housing_request.readers).to include(lessor_user.id.to_s)
      expect(applicant.readers).to include(lessor_user.id.to_s)
      expect(applicant2.readers).to include(lessor_user.id.to_s)
    end

    it 'remove lessor as readers' do
      housing_application.update!(status: 'ANNULEE BAILLEUR')
      applicant.reload
      applicant2.reload
      expect(housing_application.readers).not_to include(lessor_user.id.to_s)
      expect(housing_request.readers).not_to include(lessor_user.id.to_s)
      expect(applicant.readers).not_to include(lessor_user.id.to_s)
      expect(applicant2.readers).not_to include(lessor_user.id.to_s)
    end
  end
end
